/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <grp.h>

#define LOGPERM	(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP)

int logfd = -1;

int
initlog(const char *logfile)
{
	struct stat sbuf;
	struct group *gp;
	int fl;

	/*
	 * Open the log file, creating it if it isn't
	 * present.
	 */
	logfd = open(logfile, O_WRONLY|O_APPEND|O_CREAT,
	  LOGPERM);
	if (logfd < 0)
		return(-1);

	/*
	 * Set the close-on-exec flag.
	 */
	fl = fcntl(logfd, F_GETFD, 0);
	if ((fl < 0) ||
	  (fcntl(logfd, F_SETFD, fl|FD_CLOEXEC) < 0)) {
		close(logfd);
		logfd = -1;
		return(-1);
	}

	/*
	 * Get the group ID for group "daemon."
	 */
	gp = getgrnam("daemon");
	if (gp == NULL) {
		close(logfd);
		logfd = -1;
		errno = EINVAL;
		return(-1);
	}

	/*
	 * If the file isn't owned by the super-user or group
	 * daemon, change the file's user and group IDs.
	 */
	if (fstat(logfd, &sbuf) < 0) {
		close(logfd);
		logfd = -1;
		return(-1);
	}
	if ((sbuf.st_uid != 0) || (sbuf.st_gid != gp->gr_gid))
		fchown(logfd, 0, gp->gr_gid);
	return(0);
}
